import unohelper, uno
import ctypes
import threading
from com.sun.star.awt import XActionListener
from com.sun.star.script.provider import XScriptProvider
from com.sun.star.beans import PropertyValue
import json
import os
import os.path
from urllib.request import urlopen, urlretrieve
from urllib.parse import quote, quote_plus
import traceback
import time
from com.sun.star.task import XJobExecutor
import csv 
import glob
from array import array

import urllib

def path2url(path):
    # From http://stackoverflow.com/a/14298190, adjusted to Python 3.
    return urllib.parse.urljoin('file:', urllib.request.pathname2url(path))

def get_home_directory():
    home = os.path.expanduser("~")
    return home

localpath = get_home_directory() + os.path.sep + "OCAL" + os.path.sep
#localpath = "file://"+ os.path.sep

def get_filename_parts_from_url(url):
    fullname = url.split("/")[-1].split('#')[0].split('?')[0]
    return fullname

def download(url):
    # If the file does not already exist locally, download it.
    # Always return the full local filename.
    if not (os.path.exists(localpath)):
        os.makedirs(localpath)
    relativeFilename = get_filename_parts_from_url(url)
    absoluteFilename = localpath + relativeFilename
    if not (os.path.exists(absoluteFilename)):
        urlretrieve(url, absoluteFilename)
    return absoluteFilename

def cleanup():
    
    files = glob.glob(localpath + os.path.sep + '*')
    for f in files:
        os.remove(f)


"""
An event listener for the search button.
"""
class SearchClickedListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container, thumbs_to_show):
        self.control = control_container
        self.thumbs = thumbs_to_show

    def actionPerformed(self, actionEvent):
        searchQuery = self.control.getControl("search").getModel().Text
        if (searchQuery != ""):
            try:
                queryThread = OpenClipartQueryThread()
                queryThread.setData(self.control, 12, "load")
                queryThread.start()
            except IOError as ioe:
                showMessage("Error", "An error occured while communicating with openclipart.org\n\nYour internet connection or the server may be down.")
            except Exception as e:
                showMessage("No result", "Your query gave no results")
        

"""
An event listener for the next button.
"""

class ForwardClickedListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container, thumbs_to_show):
        self.control = control_container
        self.thumbs = thumbs_to_show

    def actionPerformed(self, actionEvent):
        try:
            queryThread = OpenClipartQueryThread()
            queryThread.setData(self.control, 12, "next")
            queryThread.start()
        except IOError as ioe:
            showMessage("Error", "An error occured while communicating with openclipart.org\n\nYour internet connection or the server may be down.")
        except Exception as e:
            showMessage("Error", "An unknown error accoured while communicating with openclipart.org")

"""
An event listener for the previous button.
"""
class BackwardClickedListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container, thumbs_to_show):
        self.control = control_container
        self.thumbs = thumbs_to_show

    def actionPerformed(self, actionEvent):
        try:
            queryThread = OpenClipartQueryThread()
            queryThread.setData(self.control, 12, "previous")
            queryThread.start()
        except IOError as ioe:
            showMessage("Error", "An error occured while communicating with openclipart.org\n\nYour internet connection or the server may be down.")
        except Exception as e:
            showMessage("Error", "An unknown error accoured while communicating with openclipart.org")

"""
An event listener for the thumbnail images.
"""
class ThumbnailClickedListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container, thumbs_to_show, index):
        self.control = control_container
        self.thumbs = thumbs_to_show
        self.index = index

    def actionPerformed(self, actionEvent):
        curData = MetadataContainer.getMetadata(self.index - 1) 
        filename = download(curData['url'])
        if (self.control.getControl('DocumentLabel').getModel().State == 1):
            insertImage(filename, curData['title'], curData['description'], curData['detail_link']) #imageName, imageName, imageDescription, linkURL
            self.control.getControl("status").getModel().Label = "Image '{0}' was inserted into document.".format(curData['title'])
        if (self.control.getControl('GalleryLabel').getModel().State == 1):
            addToGallery(filename, curData['title'])
            self.control.getControl("status").getModel().Label = "Image '{0}' was added to gallery.".format(curData['title'])

"""
A threaded class in which we will do our lookups. It is done this way for better
user experience.
"""
class OpenClipartQueryThread(threading.Thread):
    """
    The constructor. All our parameters are set her.
    """
    def __init__(self):
        # We check if a thread is already running. If so, we stop it.
        for th in threading.enumerate():
            if (th.getName() == "ocal_image_thread"):
                th.stop()
        super(OpenClipartQueryThread, self).__init__(name="ocal_image_thread")
        self._stop = threading.Event()

    """
    The next two methods are for handling threads. They are used to stop a
    thread and to check if it has been stopped.
    """
    def stop(self):
        self._stop.set()

    def stopped(self):
        return self._stop.isSet()

    def setData(self, control, thumbs_to_show, page):
        self.thumbs = thumbs_to_show
        self.page = page
        self.control = control

    """
    This is where the action takes place.
    """
    def run(self):
        try:
            self.control.getControl("status").getModel().Label = "Searching for images..."
            i = 1
            while (i <= self.thumbs and not self._stop.isSet()):
                self.control.getControl("image" + str(i)).getModel().Enabled = False
                self.control.getControl("image" + str(i)).getModel().ImageURL = ""
                self.control.getControl("title" + str(i)).getModel().Label = ""
                i += 1
            if (self.page == "load"):
                searchQuery = quote(str(self.control.getControl("search").getModel().Text))
                querySort = quote(str(self.control.getControl("Sort").getModel().Text))
                data = Parser.load(str(searchQuery),str(querySort), 12, True)['payload']
                #Handling zero result
                if (Parser.total_number_of_records==0):
                    self.control.getControl("status").getModel().Label = "Sorry, no matching clipart was found."
                    return
                if (Parser.has_next_page()):
                    self.control.getControl("nextButton").getModel().Enabled = True
                else:
                    self.control.getControl("nextButton").getModel().Enabled = False
                self.control.getControl("previousButton").getModel().Enabled = False
            elif (self.page == "next"):
                data = Parser.next_page()['payload']
                if (Parser.has_next_page()):
                    self.control.getControl("nextButton").getModel().Enabled = True
                else:
                    self.control.getControl("nextButton").getModel().Enabled = False
                self.control.getControl("previousButton").getModel().Enabled = True
            elif (self.page == "previous"):
                data = Parser.previous_page()['payload']
                self.control.getControl("nextButton").getModel().Enabled = True
                if (Parser.has_previous_page()):
                    self.control.getControl("previousButton").getModel().Enabled = True
                else:
                    self.control.getControl("previousButton").getModel().Enabled = False
            if (not data is None):
                i = 1
                """
                We clear the tiles.
                """
                while (i <= self.thumbs and not self._stop.isSet()):
                    self.control.getControl("image" + str(i)).getModel().Enabled = False
                    self.control.getControl("image" + str(i)).getModel().ImageURL = ""
                    self.control.getControl("title" + str(i)).getModel().Label = ""
                    i += 1
                i = 1
                """
                We refresh the metadata matrix if it contains data.
                """
                if (MetadataContainer.hasData()):
                    MetadataContainer.refresh()
                currentText = str(Parser.current_page()) + " of " + str(Parser.total_pages())
                self.control.getControl("currentText").getModel().Label = str(currentText)
                if (Parser.current_page() == Parser.total_pages()):
                    number_of_thumbs = Parser.total_records() - ((Parser.total_pages() - 1) * self.thumbs)  
                else:
                    number_of_thumbs = self.thumbs
                while (i <= number_of_thumbs and not self._stop.isSet()):
                    curImage = data[i-1]
                    png_url = curImage['svg']['png_thumb']
                    png_file = png_url[(png_url.rfind('/') + 1):]
                    thumb_url = "https://openclipart.org/image/100px/svg_to_png/" + png_file
                    localFilename = download(thumb_url)
                    fileUrl = path2url(localFilename)  # "file://" + local
                    self.control.getControl("status").getModel().Label = "Getting " + localFilename #thumb_url      #Debugging 
                    imageButton = self.control.getControl("image" + str(i)).getModel()
                    imageButton.Enabled = True
                    imageButton.ImageURL = fileUrl
                    imageButton.HelpText = "Click to insert " + curImage['title']
                    self.control.getControl("title" + str(i)).getModel().Label = curImage['title']
                    i += 1
                    """
                    We assign the meta data.
                    """
                    metadata = {
                        "url": curImage['svg']['url'],
                        "title": curImage['title'],
                        "uploader": curImage['uploader'],
                        #"drawn_by": curImage['drawn_by'],
                        "description": curImage['description'],
                        "detail_link": curImage['detail_link'],
                        "created": curImage['created']
                    }
                    MetadataContainer.addData(metadata)
            else:
                showMessage("No result", "Your search for '" + str(self.control.getControl("search").getModel().Text) + "' gave no results.")
            self.control.getControl("status").getModel().Label = ""
        except IOError as ioe:
            showMessage("Error", "An error occurred while communicating with openclipart.org:\n" + ioe.strerror + "\n\nYour internet connection or the server may be down.")
        except ValueError as ve:
            showMessage("No result", "Your search for '" + str(self.control.getControl("search").getModel().Text) + "' gave no results.")
        except Exception as e:
            showMessage("Error", "An unknown error occurred while communicating with openclipart.org:\n\n" + traceback.format_exc() + "\n\n" + "TYPE:" + str(type(e)))

"""
An event listener for the previous button.
"""
class MessageBoxListener( unohelper.Base, XActionListener ):
    def __init__(self, control_container):
        self.control = control_container
        
    def actionPerformed(self, actionEvent):
        self.control.endExecute()


def addToGallery(url, title):
    ctx = uno.getComponentContext()
    oDoc = XSCRIPTCONTEXT.getDocument()
    smgr = uno.getComponentContext().ServiceManager
    inDex = -1
    galleryname= "OpenClipart.org"
    oGalleryThemes = createUnoService('com.sun.star.gallery.GalleryThemeProvider')
    oImage = oDoc.createInstance('com.sun.star.drawing.GraphicObjectShape')
    if not oGalleryThemes.hasByName(galleryname):
        oGalleryThemes.insertNewByName(galleryname)
    theme = oGalleryThemes.getByName(galleryname)
    response = theme.insertURLByIndex ("file://"+url,inDex)
    clipart = theme.getByIndex(0)
    clipart.Title = title
    if (response != None):
        # We show a debug message in status field
        self.control.getControl("status").getModel().Label = "Image '{0}' was added to gallery.".format(curData['title'])
   
def insertImage(imageName, imageTitle, imageDescription, linkURL):
    ctx = XSCRIPTCONTEXT.getComponentContext()
    smgr = ctx.getServiceManager()
    doc = XSCRIPTCONTEXT.getDocument()
    document = XSCRIPTCONTEXT.getDocument().getCurrentController().getFrame()
    desktop = XSCRIPTCONTEXT.getDesktop()

    dispatcher = smgr.createInstanceWithContext("com.sun.star.frame.DispatchHelper", ctx)

    args = [ PropertyValue() for i in range(4)]
    args[0].Name="FileName"
    args[0].Value="file://"+imageName
    args[1].Name="FilterName"
    args[1].Value="<All formats>"
    args[2].Name = "AsLink"
    args[2].Value = False
    args[3].Name="Style"
    args[3].Value="Graphics"
    
    dispatcher.executeDispatch(document, ".uno:InsertGraphic", "", 0, tuple(args))
    selection = doc.getCurrentSelection()
    selection.Title = imageTitle
    selection.Name = imageTitle
    selection.Description = imageDescription
    selection.HyperLinkURL = linkURL
    

 
def createUnoService(serviceName):
    sm = uno.getComponentContext().ServiceManager
    return sm.createInstanceWithContext(serviceName, uno.getComponentContext())    


def debug(message):
    showMessage("Debug", message)

"""
Here we define a message box - mostly for debugging.
"""
def showMessage(title, text):
    ctx = uno.getComponentContext()
    smgr = ctx.ServiceManager

    # create the dialog model and set the properties 
    dialogModel = smgr.createInstanceWithContext("com.sun.star.awt.UnoControlDialogModel", ctx)
    dialogModel.Width = 600
    dialogModel.Height = 400
    dialogModel.PositionX = 0
    dialogModel.PositionY = 15
    dialogModel.Title = title

    if False:
        # Label to hold the text
        labelModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
        labelModel.PositionX = 3
        labelModel.PositionY = 3
        labelModel.Width  = 590
        labelModel.Height = 300
        labelModel.Name = "msgText"
        labelModel.TabIndex = 0
        labelModel.Label = text
        labelModel.MultiLine = True
    else:
        # Text box to hold the text, so the user can copy from it
        labelModel = dialogModel.createInstance("com.sun.star.awt.UnoControlEditModel" ); 
        labelModel.PositionX = 3
        labelModel.PositionY = 3
        labelModel.Width  = 590
        labelModel.Height = 300
        labelModel.Name = "msgText"
        labelModel.TabIndex = 0
        labelModel.Text = text
        labelModel.MultiLine = True
        labelModel.ReadOnly = True

    # create the button model model and set the properties 
    buttonModel = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel" ); 
    buttonModel.PositionX = 25
    buttonModel.PositionY = 310
    buttonModel.Width  = 50
    buttonModel.Height = 15 
    buttonModel.TabIndex = 0
    buttonModel.Name = 'okButton'
    buttonModel.Label = 'OK'

    # insert the control models into the dialog model 
    dialogModel.insertByName("msgText", labelModel)
    dialogModel.insertByName("okButton", buttonModel)

    # create the dialog control and set the model 
    controlContainer = smgr.createInstanceWithContext("com.sun.star.awt.UnoControlDialog", ctx); 
    controlContainer.setModel(dialogModel); 

    controlContainer.getControl("okButton").addActionListener(MessageBoxListener(controlContainer))

    # create a peer 
    toolkit = smgr.createInstanceWithContext("com.sun.star.awt.ExtToolkit", ctx);

    controlContainer.setVisible(False);       
    controlContainer.createPeer(toolkit, None);


    # execute it
    controlContainer.execute()

    # dispose the dialog 
    controlContainer.dispose()

"""
Here we define the dialog to display the image thumbnails.
"""
def createDialog(Event="default"):
    script_dir = os.path.dirname(__file__)[:-6]
    #self.control.getControl("status").getModel().Label = "Cleaning up temporary folder"
    cleanup()    
    imageModels = []
    labelModels = []
    ctx = uno.getComponentContext()
    smgr = ctx.ServiceManager

    # create the dialog model and set the properties 
    dialogModel = smgr.createInstanceWithContext("com.sun.star.awt.UnoControlDialogModel", ctx)

    dialogModel.Width = 223
    dialogModel.Height = 259
    dialogModel.PositionX = 17
    dialogModel.PositionY = 30
    dialogModel.Title = 'OpenClipart.org'

    # create the label model and set the properties 
    labelModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    labelModel.PositionX = 8
    labelModel.PositionY = 24 
    labelModel.Width  = 22 
    labelModel.Height = 11 
    labelModel.Name = "SearchLabel" 
    labelModel.Label = "Search: "

    # create the edit model and set the properties 
    editModel = dialogModel.createInstance( "com.sun.star.awt.UnoControlEditModel" ); 
    editModel.PositionX = 30
    editModel.PositionY = 23
    editModel.Width  = 100
    editModel.Height = 12 
    editModel.TabIndex = 0
    editModel.Name = 'search'
    editModel.Text = ''

    # create the button model model and set the properties 
    buttonModel = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel" ); 
    buttonModel.PositionX = 200
    buttonModel.PositionY = 22
    buttonModel.Width  = 18
    buttonModel.Height = 15 
    buttonModel.TabIndex = 2
    buttonModel.ImageURL= script_dir + "icons/g1772.png"
    buttonModel.HelpText="Click to search"
    buttonModel.Name = 'Search'
    buttonModel.DefaultButton = 'True'

    # create the label model and set the properties 
    countModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    countModel.PositionX = 39
    countModel.PositionY = 245
    countModel.Width  = 43 
    countModel.Height = 9 
    countModel.Name = "Counter" 
    countModel.Label = "Page:"

    # create the label model and set the properties 
    currentModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    currentModel.PositionX = 53
    currentModel.PositionY = 245
    currentModel.Width  = 50
    currentModel.Height = 11
    currentModel.Name = "CurrentText" 

    # create the button model model and set the properties 
    previousModel = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel" ); 
    previousModel.PositionX = 8
    previousModel.PositionY = 244
    previousModel.Width  = 12
    previousModel.Height = 12 
    previousModel.TabIndex = 3
    previousModel.Name = 'PreviousButton'
    previousModel.ImageURL= script_dir + "icons/path8643.png"
    previousModel.Enabled = False

    # create the button model model and set the properties 
    nextModel = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel" ); 
    nextModel.PositionX = 23
    nextModel.PositionY = 244
    nextModel.Width  = 12
    nextModel.Height = 12 
    nextModel.TabIndex = 4
    nextModel.Name = 'NextButton'
    nextModel.ImageURL= script_dir + "icons/path8644.png"
    nextModel.Enabled  = False

    # create the label model and set the properties 
    sortlabelModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    sortlabelModel.PositionX = 135
    sortlabelModel.PositionY = 24
    sortlabelModel.Width  = 17
    sortlabelModel.Height = 11
    sortlabelModel.Align = 2
    sortlabelModel.Name = 'SortLabel'
    sortlabelModel.Label = 'Sort: '
    
# create the Comvobox model and set the properties 
    sortModel = dialogModel.createInstance("com.sun.star.awt.UnoControlComboBoxModel" );
    sortComboList = ('downloads','favorites','date')
    sortModel.PositionX = 154
    sortModel.PositionY = 23
    sortModel.Width = 40
    sortModel.Height = 12
    sortModel.Name = 'Sort'
    sortModel.TabIndex = 1
    sortModel.Text = sortComboList[0]
    sortModel.Dropdown = 1
    sortModel.StringItemList = sortComboList

#Create the checkbox models and set the properties    
    gallerylabelModel = dialogModel.createInstance("com.sun.star.awt.UnoControlCheckBoxModel" ); 
    gallerylabelModel.PositionX = 170
    gallerylabelModel.PositionY = 245
    gallerylabelModel.Width  = 50
    gallerylabelModel.Height = 11
    gallerylabelModel.State = 0
    gallerylabelModel.Enabled = 1
    gallerylabelModel.Name = 'GalleryLabel'
    gallerylabelModel.Label = 'Add to gallery?'   
    gallerylabelModel.HelpText ='Check if you want the clipart to be added to the Gallery'
    
    documentlabelModel = dialogModel.createInstance("com.sun.star.awt.UnoControlCheckBoxModel" ); 
    documentlabelModel.PositionX = 100
    documentlabelModel.PositionY = 245
    documentlabelModel.Width  = 75
    documentlabelModel.Height = 11
    documentlabelModel.State = 1
    documentlabelModel.Enabled = 1
    documentlabelModel.Name = 'DocumentLabel'
    documentlabelModel.Label = 'Insert into document?'   
    documentlabelModel.HelpText ='Check if you want the clipart to be inserted into the document'
    
    """
    We define the image models.
    """
    rows = 3
    cols = 4
    startPosX = 7
    startPosY = 55
    #tabindex=20
    image = 0
    thumbs_to_show = rows * cols

    for row in range (1 ,rows+1):

        	   # 55 | 119 | 183  
        for col  in range(1, cols+1):
            curImage = dialogModel.createInstance("com.sun.star.awt.UnoControlButtonModel");
            curLabel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel");
            curImage.PositionY = startPosY+(row-1)*64
            image = image + 1
            tabIndex = image + 20
            curImage.PositionX = startPosX+(col-1)*54       # 7 | 61 | 115 | 169 
            curImage.TabIndex = tabIndex
            curImage.Enabled = False
            #curImage.ImageScaleMode = "com.sun.star.awt.ImageScaleMode.ISOTROPIC"
            curLabel.PositionY = curImage.PositionY + 52
            curLabel.PositionX = curImage.PositionX
            curImage.Width = 50
            curImage.Height = 50
            curImage.Name = 'image' + str(image)
            curImage.HelpText = ''
            dialogModel.insertByName('image' + str(image), curImage)
            curLabel.Width = 50
            curLabel.Height = 10
            curLabel.Name = 'title' + str(image)
            dialogModel.insertByName('title' + str(image), curLabel)
            

    # We create a status label - mostly for debugging
    statusModel = dialogModel.createInstance("com.sun.star.awt.UnoControlFixedTextModel" ); 
    statusModel.PositionX = 8
    statusModel.PositionY = 40
    statusModel.Width  = 210 
    statusModel.Height = 10
    statusModel.BackgroundColor = 0xbbbbbb
    statusModel.MultiLine = False
    statusModel.Name = "status"
    statusModel.Label = ""

    # insert the control models into the dialog model 
    dialogModel.insertByName( "searchButton", buttonModel)
    dialogModel.insertByName( "searchLabel", labelModel)
    dialogModel.insertByName( "search", editModel)
    dialogModel.insertByName( "nextButton", nextModel)
    dialogModel.insertByName( "previousButton", previousModel)
    dialogModel.insertByName( "counter", countModel)
    dialogModel.insertByName( "currentText", currentModel)
    dialogModel.insertByName( "status", statusModel)
    dialogModel.insertByName( "Sort", sortModel)
    dialogModel.insertByName( "SortLabel", sortlabelModel)
    dialogModel.insertByName( "GalleryLabel", gallerylabelModel)
    dialogModel.insertByName( "DocumentLabel", documentlabelModel)
    #dialogModel.insertByName( "scrollBar", scrollBar)

    # create the dialog control and set the model 
    controlContainer = smgr.createInstanceWithContext("com.sun.star.awt.UnoControlDialog", ctx); 
    controlContainer.setModel(dialogModel); 

    # add the action listener
    i = 1
    while (i <= thumbs_to_show):
        controlContainer.getControl('image'+str(i)).addActionListener(ThumbnailClickedListener(controlContainer, thumbs_to_show, i))
        i += 1

    controlContainer.getControl("searchButton").addActionListener(SearchClickedListener(controlContainer, thumbs_to_show))
    controlContainer.getControl("nextButton").addActionListener(ForwardClickedListener(controlContainer, thumbs_to_show))
    controlContainer.getControl("previousButton").addActionListener(BackwardClickedListener(controlContainer, thumbs_to_show))

    # create a peer 
    toolkit = smgr.createInstanceWithContext("com.sun.star.awt.ExtToolkit", ctx);

    controlContainer.setVisible(False);       
    controlContainer.createPeer(toolkit, None);

    # execute it
    controlContainer.execute()

    # dispose the dialog 
    controlContainer.dispose()

class Parser(object):
    #Class variables
    query_prefix = ""
    total_number_of_pages = 0
    total_number_of_api_pages = 0
    total_number_of_records = 0
    results_per_page = 0
    current_page_nr = 0
    next_page_nr = 0
    previous_page_nr = 0
    
    """
    A static method that initializes the load from openclipart.org.
    Furthermore control variable are set for the future readings.

    PARAMETERS
    query*              the search query from the user
    results_per_page    the number of results to be shown per page
                        this far we only handle values of tenths (10, 20 etc.)
    return_data         boolean that specifies whether to return the data or not
    
    * = required parameter
    
    RETURNS
    On success:         returns 1 or (if requested) the JSON data.
    On fail:            returns None. 
    """
    @classmethod
    def load(Parser, query, querysort, results_per_page=12, return_data=False):
    # def load(Parser, query, querysort, results_per_page=10, return_data=False):
        """
        We do the first initial load and set the appropriate variables.
        """

        """ We set the prefix for the query URL for later use. """
        Parser.query_prefix = 'https://openclipart.org/search/json/?query=' + query  + '&amount=12&sort=' + querysort
        
        Parser.results_per_page = results_per_page
        Parser.current_page_nr = 1

        """ We parse the initial data load. """
        data = Parser.read(1)

        Parser.total_number_of_api_pages = data['info']['pages']
        Parser.total_number_of_pages = data['info']['pages'] #/ (Parser.results_per_page / 12)
        Parser.total_number_of_records = data['info']['results']
        
        """ We set the initial pagination variables- """
        Parser.current_page_nr = 1
        Parser.previous_page_nr = 0
        #Parser.next_page_nr = 2   #Was 1

        """
        When ocal receives a query with no matches it will throw a mysql
        warning, which of course is not valid JSON. Thus we have to check
        if our data object is None and if so we will return the same.
        """
        if (data is None):
            return None
        
        """
        Otherwise we proceed and check if the object contains any results. The
        mysql warning may be fixed at a moment and we have to look out for that.
        """
        if (data['info']['results'] == '0'):
            return None

        if (Parser.total_number_of_pages > 1):
            Parser.next_page_nr = 2

        if (return_data):
            return data
        else:
            return 1
        

    """
    A static method that reads data from a given page.

    PARAMETERS
    page_number*        the page number to show
    
    * = required parameter
    
    RETURNS
    The JSON data or None of no results were found.
    """
    @classmethod
    def read(Parser, page_number):
        
        """
        As for now the openclipart.org API does not have a reseults_per_page
        parameter and we thus have to accomolate the received data.
        """
        url = Parser.query_prefix + "&page=" + str(page_number)
        content_file = urlopen(url)
        json_string = content_file.read().decode('ISO-8859-1')

        if (json_string is None):
            return None

        """
        We trim some incorrect JSON from the OCAL API.
        """
        json_string = json_string.replace(" : null,", " : \"none\",")
        json_string = json_string.replace("},\n]", "}]")

        if (Parser.results_per_page > 12):
            showMessage("Debug","In the wrong loop. Results: "+ Parser.results_per_page)
            """
            In this loop we concatenate results for the requested number
            of pages.
            """
            tmp_data = json_string[:json_string.rfind('	]\n\n}')]
            i = 2
            while (i <= (Parser.results_per_page / 12)):
                if (page_number):
                    page = str(i)
                else:
                    page = str(i + (page_number * (Parser.results_per_page / 12)))
                cur_file = urlopen(Parser.query_prefix + "&page=" + page)
                cur_json = cur_file.read().decode('ISO-8859-1')
                cur_json = cur_json.replace(" : null,", " : \"none\",")
                cur_json = cur_json.replace("},\n]", "}]")
                tmp_data += cur_json[
                    cur_json.rfind('"payload" : [')
                    :
                    cur_json.rfind('	]\n\n}')
                ]
                i += 1
            """
            openclipart.org does not return entirely clean JSON, so we need to tell
            the json library to be less strict.
            Furthermore the last page always returns bad JSON, so we need make a
            workaround for that particular case. The problem is a JSON list endning
            with a comma, so we check if that is the case and remove it if so.
            """
            data = json.loads(tmp_data, strict=False)
        else:
            data = json.loads(json_string, strict=False)
        Parser.current_page_nr = page_number
        if (Parser.current_page_nr > 0):
            Parser.previous_page_nr = Parser.current_page_nr - 1
        if (Parser.current_page_nr < Parser.total_number_of_pages):
            Parser.next_page_nr = Parser.current_page_nr + 1
        return data

    """
    A static method that returns the total number of pages.
    """
    @classmethod
    def total_pages(Parser):
        return int(Parser.total_number_of_pages)

    """
    A static method that returns the total number of records.
    """
    @classmethod
    def total_records(Parser):
        return int(Parser.total_number_of_records)

    """
    A static method that returns the current page number.
    """
    @classmethod
    def current_page(Parser):
        return int(Parser.current_page_nr)

    """
    A static method that returns the data of the next page if any. Otherwise
    it returns None.
    """
    @classmethod
    def next_page(Parser):
        if (Parser.next_page_nr > 0):
            page_to_return = Parser.next_page_nr
            if (Parser.next_page_nr < Parser.total_number_of_pages):
                Parser.next_page_nr += 1
                Parser.current_page_nr += 1
                Parser.previous_page_nr += 1
            else:
                Parser.next_page_nr = 0
            return Parser.read(page_to_return)
        else:
            return None

    """
    A static method that returns the data of the previous page if any. Otherwise
    it returns None.
    """
    @classmethod
    def previous_page(Parser):
        if (Parser.previous_page_nr < Parser.total_number_of_pages):
            page_to_return = Parser.previous_page_nr
            if (Parser.previous_page_nr > 0):
                Parser.previous_page_nr -= 1
                Parser.current_page_nr -= 1
                Parser.next_page_nr -= 1
            else:
                Parser.previous_page_nr = 0
            return Parser.read(page_to_return)
        else:
            return None

    """
    A static method that returns if there are previous pages to show
    """
    @classmethod
    def has_previous_page(Parser):
        return Parser.previous_page_nr > 0

    """
    A static method that returns if there are more pages to show
    """
    @classmethod
    def has_next_page(Parser):
        return Parser.next_page_nr > 0
    


class MetadataContainer(object):
    metaMatrix = []

    """
    A static method that appends metadata to the .

    PARAMETERS
    dataTuple                   a tuple that represents the metadata of a give
                                image.
    """
    @classmethod
    def addData(MetadataContainer, dataTuple):
        MetadataContainer.metaMatrix.append(dataTuple)

    """
    A static method that returns the entire metadata matrix.

    RETURNS
    On success:         returns a copy of the metadata matrix.
    On fail:            returns None. 
    """
    @classmethod
    def getMetadataMatrix(MetadataContainer):
        if (len(MetadataContainer.metaMatrix) > 0):
            return MetadataContainer.metaMatrix
        else:
            return None

    """
    A static method that returns the metadata of a specific element.

    PARAMETERS
    index               the of the metadata to be read (0 based)
    
    RETURNS
    On success:         returns the metadata tuple.
    On fail:            returns None. 
    """
    @classmethod
    def getMetadata(MetadataContainer, index):
        return MetadataContainer.metaMatrix[index]

    """
    A static method used to refresh the metadata matrix.
    """
    @classmethod
    def refresh(MetadataContainer):
        MetadataContainer.metaMatrix = []

    """
    A static method used to check if the matrix contains elements.
    """
    @classmethod
    def hasData(MetadataContainer):
        return (len(MetadataContainer.metaMatrix) > 0)

g_ImplementationHelper = unohelper.ImplementationHelper()

g_exportedScripts = createDialog,
